package pages
{
	import flash.events.Event;
	import flash.geom.Rectangle;
	
	import jp.progression.casts.CastSprite;
	import jp.progression.commands.display.AddChild;
	import jp.progression.commands.display.RemoveAllChildren;
	
	import org.libspark.pv3d.Metasequoia;
	import org.papervision3d.core.math.Matrix3D;
	import org.papervision3d.materials.BitmapFileMaterial;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.view.BasicView;
	
	import tools.Detector;
	import tools.VideocameraTool;
	
	public class TopPage extends CastSprite
	{
		private var _detector:Detector;
		private var _view:BasicView;
		private var _tool:VideocameraTool;
		private var _viewScaleX:Number;
		private var _viewScaleY:Number;
		private static const _SCALE_VIEW:Number = 0.8;
		private static const _RECT_MIN:Number = 59;
		private static const _RECT_MAX:Number = 103;
		private static const _Z_RENGE:Number = 20;
		private var _renge:Number;
		private static const _PI:Number = Math.PI / 180;
		
		private var _cameraMatrix:Matrix3D;
		public function TopPage(initObject:Object=null)
		{
			super(initObject);
		}

		private function _detectorUpdateHandler(e:Event):void
		{
			var rect:Rectangle = _detector.rect;
			if(!(!rect)){
				var rX:Number = (rect.x / VideocameraTool.VIDEO_SCALE) - ((_tool.video.width - (rect.width / VideocameraTool.VIDEO_SCALE)) * 0.5);
				var rY:Number = (rect.y / VideocameraTool.VIDEO_SCALE) - ((_tool.video.height - (rect.height / VideocameraTool.VIDEO_SCALE)) * 0.5);
				var rZ:Number = (rect.width - _RECT_MIN) * _renge;
				var cX:Number = rX * _viewScaleX * _SCALE_VIEW;
				var cY:Number = -rY * _viewScaleY * _SCALE_VIEW;
					
				_view.camera.zoom = rZ + 40;

				var matX:Matrix3D = Matrix3D.rotationMatrix(0, 1, 0, cX * _PI);
				var matY:Matrix3D = Matrix3D.rotationMatrix(1, 0, 0, cY * _PI);
				var mat:Matrix3D = Matrix3D.multiply(matX, matY);
				_view.camera.transform = Matrix3D.multiply(mat, _cameraMatrix);

			}
		}

		protected override function atCastAdded():void
		{
			_tool = VideocameraTool.getInstance();
			_viewScaleX = stage.stageWidth / _tool.video.width;
			_viewScaleY = stage.stageHeight / _tool.video.height;
			_detector = new Detector();
			_view = new BasicView();
			_renge = _Z_RENGE / (_RECT_MAX - _RECT_MIN);
			
			var plane:Plane = new Plane(new BitmapFileMaterial("http://utweb.jp/samples/kao2/back.jpg"), 640, 480, 2);
			//plane.z = 200;
			_view.scene.addChild(plane);
			
			var genki:Metasequoia = new Metasequoia();
			genki.load("http://utweb.jp/samples/kao2/genkikun.mqo");
			genki.z = -200;
			_view.scene.addChild(genki);
			//_view.camera.target = genki;
			_cameraMatrix = Matrix3D.clone(_view.camera.transform);
			addCommand(
				new AddChild(this, _detector),
				new AddChild(this, _view),
				function():void{
					_view.startRendering();
					_detector.addEventListener(Detector.RECTANGLE_UPDATE, _detectorUpdateHandler);
				}
			);
		}

		protected override function atCastRemoved():void
		{
			addCommand(
				new RemoveAllChildren(this),
				function():void{
					_tool = null;
					_detector.removeEventListener(Detector.RECTANGLE_UPDATE, _detectorUpdateHandler);
					_detector = null;
					_view.stopRendering();
					_view = null;
				}
			);
		}

	}
}